if( !require('tseries') ){
    install.packages('tseries', dependencies=TRUE, repos='http://cran.rstudio.com/')
}
library(tseries)

save_plots = FALSE

set.seed(0)

source('./utils.R')

ticker = "IBM"
data = get.hist.quote(instrument=ticker,quiet=TRUE)
log_rets = diff(log(data$Close))
rets = log_rets[(length(log_rets)-999):length(log_rets)] # get a sample of 1000 returns
rets = as.vector( rets ) 

S = 10000
alpha = 0.025

## Part (a): Normal assumption
## 
m = mean(rets)
s = sd(rets)
VaR_norm_pt_a = VaR_norm(alpha,m,s,S)
print(sprintf('VaR_norm= %f', VaR_norm_pt_a))

## Part (b): Nonparametric
##
t = quantile( rets, probs=alpha )
VaR_np = -S * t
print(sprintf('VaR_np= %f', VaR_np))

## Part (c) a QQ-plot
##
if( save_plots ){ postscript("../../WriteUp/Graphics/Chapter11/chap_11_prob_4_pt_c_norm_qqplot.eps", onefile=FALSE, horizontal=FALSE) }
qqnorm(rets)
qqline(rets)
grid()
if( save_plots ){ dev.off() }

## Part (d): estimate left-tail index a (using the Hill estimator):
##
sorted_rets = sort( rets )
Ys = abs(sorted_rets)
a_hill = rep( 0, length(Ys) ) 
for( nc in 1:length(Ys) ){
    c = Ys[nc]
    a_hill[nc] = nc / sum( log(Ys[1:nc]/c) )
}
a = mean( a_hill[50:100] )

if( save_plots ){ postscript("../../WriteUp/Graphics/Chapter11/chap_11_prob_4_pt_d_hill_plot.eps", onefile=FALSE, horizontal=FALSE) }
plot( 1:length(Ys), a_hill, xlim=c(2,200), ylim=c(-0.1,4), xlab="nc", ylab="a", main="a^{Hill} estimate" )
abline(h=a, col='red')
grid()
if( save_plots ){ dev.off() }

VaR_pt_d = VaR_norm_pt_a * ( 0.025 / 0.0025 )^(1/a)
print(sprintf('a_hat= %f; VaR_pt_d= %f', a, VaR_pt_d))
